import axios from "axios";
import Router from "@/router";
import { Message, Notification } from "@arco-design/web-vue";
import { useSystemStore } from "@/store/modules/systemStore";
import moment from 'moment-timezone';
import { fn_logout } from "@/utils";
import error from "@/i18n/zh/error";

const axiosInstance = axios.create({
  // baseURL: `${import.meta.env.PROD? import.meta.env.VITE_PRO_PATH : ''}/api`,
  baseURL: import.meta.env.BASE_URL + "api",
  timeout: 300000,
});

const requestState = {
  // 得到正确响应
  success: [200],
  // token 跳转
  beOverdue: [886],
  // 没有访问权限
  NotAccessRight: [500],
  // 异常 code
  exception: [400],
};

const pathArr = ["/api/admin/system/login", "/api/logout"];
axiosInstance.interceptors.request.use(
  (config) => {
    const systemStore = useSystemStore();
    systemStore.localLoading(true);

    // 在发送请求之前做些什么
    if (!pathArr.includes(config.url)) {
      const token = localStorage.getItem("token");
      if (token && config.headers) {
        config.headers["Authorization"] = token;
      }
    }
    return config;
  },
  (err) => {
    const systemStore = useSystemStore();
    systemStore.localLoading();
    Promise.reject(err);
  }
);

// 响应拦截器
axiosInstance.interceptors.response.use(
  (res) => {
    const systemStore = useSystemStore();
    systemStore.localLoading();
    const { code, data, message: msg } = res.data;
    // 成功
    if (code === 200) {
      const dataList = {
        code: code,
        message: res.data.msg,
        data: DataViewsProvider(data)
      };
      return Promise.resolve(dataList);
    }

    // 服务端错误信息
    if (requestState.NotAccessRight.includes(code)) {
      Notification.warning({
        title: "系统信息",
        content: msg,
      });
      return Promise.reject(msg);
    }

    // 异常 code
    if (requestState.exception.includes(code)) {
      Notification.warning({
        title: "系统提示",
        content: msg,
      });
      return Promise.reject(msg);
    }

    return Promise.resolve(data);
  },
  (err) => {
    const systemStore = useSystemStore();
    systemStore.localLoading();
    const res = err["response"];
    if (err.code === "ERR_CANCELED") {
      console.log("请求中断");
      return;
    }
    const msg = err.response?.data ? err.response.data.message : "";
    Notification.warning({
      title: "系统信息",
      content: msg
    });
    // token 失效
    if (requestState.beOverdue.includes(err.status)) {
      fn_logout(Router);
      // 直接终止请求
      return undefined;
    }
    Promise.reject(err);
  }
);


function timeLoadTemeYear(value) {
  if (!value) return;
  const systemStore = useSystemStore();
  const date = moment(value, moment.ISO_8601);
  if (!date.isValid()) return null
  // 使用用户所在时区转换时间
  if (systemStore.getUserIp && systemStore.getUserIp.timezone) {
    const localDate = date.tz(systemStore.getUserIp.timezone);
    const time = localDate.format('YYYY-MM-DD HH:mm:ss');
    return time;
  } else {
    return null;
  }
}

// 转换数据格式 
function DataViewsProvider(data) {
  if (!Array.isArray(data)) return data

  if (data && data.records) {
    return { ...data, records: data.records.map(item => ({ ...item, createdAt: timeLoadTemeYear(item.createdAt || false), updatedAt: timeLoadTemeYear(item.updatedAt || false) ,endDate: timeLoadTemeYear(item.endDate || false) })) }
  }else{
    return data.map(item => ({...item, createdAt: timeLoadTemeYear(item.createdAt || false), updatedAt: timeLoadTemeYear(item.updatedAt || false) }))
  }
}

export default axiosInstance;